@@ -17,7 +17,7 @@ import asyncio
 import logging
 import os
 from asyncio.exceptions import CancelledError
-from typing import Any, Dict, List, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 
 import yaml
 
@@ -36,6 +36,8 @@ from ansible_rulebook.websocket import (
     send_event_log_to_websocket,
 )
 
+from .exception import InventoryNeededException, RulebookNotFoundException
+
 
 class NullQueue:
     async def put(self, _data):
@@ -43,6 +45,7 @@ class NullQueue:
 
 
 logger = logging.getLogger(__name__)
+INVENTORY_ACTIONS = ("run_playbook", "run_module")
 
 
 # FIXME(cutwater): Replace parsed_args with clear interface
@@ -60,9 +63,9 @@ async def run(parsed_args: argparse.ArgumentParser) -> None:
             int(parsed_args.id), parsed_args.websocket_address
         )
     else:
-        inventory = {}
+        inventory = ""
         variables = load_vars(parsed_args)
-        rulesets = load_rulebook(parsed_args)
+        rulesets = load_rulebook(parsed_args, variables)
         if parsed_args.inventory:
             inventory = load_inventory(parsed_args.inventory)
         project_data_file = parsed_args.project_tarball
@@ -74,7 +77,10 @@ async def run(parsed_args: argparse.ArgumentParser) -> None:
 
     logger.info("Starting sources")
     tasks, ruleset_queues = spawn_sources(
-        rulesets, variables, [parsed_args.source_dir]
+        rulesets,
+        variables,
+        [parsed_args.source_dir],
+        parsed_args.shutdown_delay,
     )
 
     logger.info("Starting rules")
@@ -136,7 +142,9 @@ def load_vars(parsed_args) -> Dict[str, str]:
 
 
 # TODO(cutwater): Maybe move to util.py
-def load_rulebook(parsed_args) -> List[RuleSet]:
+def load_rulebook(
+    parsed_args: argparse.ArgumentParser, variables: Optional[Dict] = None
+) -> List[RuleSet]:
     if not parsed_args.rulebook:
         logger.debug("Loading no rules")
         return []
@@ -147,24 +155,32 @@ def load_rulebook(parsed_args) -> List[RuleSet]:
         with open(parsed_args.rulebook) as f:
             data = yaml.safe_load(f.read())
             Validate.rulebook(data)
-            return rules_parser.parse_rule_sets(data)
+            if variables is None:
+                variables = {}
+            rulesets = rules_parser.parse_rule_sets(data, variables)
     elif has_rulebook(*split_collection_name(parsed_args.rulebook)):
         logger.debug(
             "Loading rules from a collection %s", parsed_args.rulebook
         )
-        return rules_parser.parse_rule_sets(
+        rulesets = rules_parser.parse_rule_sets(
             collection_load_rulebook(
                 *split_collection_name(parsed_args.rulebook)
             )
         )
     else:
-        raise Exception(f"Could not find ruleset {parsed_args.rulebook}")
+        raise RulebookNotFoundException(
+            f"Could not find rulebook {parsed_args.rulebook}"
+        )
+
+    validate_actions(rulesets, parsed_args)
+    return rulesets
 
 
 def spawn_sources(
     rulesets: List[RuleSet],
     variables: Dict[str, Any],
     source_dirs: List[str],
+    shutdown_delay: float,
 ) -> Tuple[List[asyncio.Task], List[RuleSetQueue]]:
     tasks = []
     ruleset_queues = []
@@ -177,8 +193,25 @@ def spawn_sources(
                     source_dirs,
                     variables,
                     source_queue,
+                    shutdown_delay,
                 )
             )
             tasks.append(task)
         ruleset_queues.append(RuleSetQueue(ruleset, source_queue))
     return tasks, ruleset_queues
+
+
+def validate_actions(
+    rulesets: List[RuleSet], parsed_args: argparse.ArgumentParser
+) -> None:
+    for ruleset in rulesets:
+        for rule in ruleset.rules:
+            for action in rule.actions:
+                if (
+                    action.action in INVENTORY_ACTIONS
+                    and parsed_args.inventory is None
+                ):
+                    raise InventoryNeededException(
+                        f"Rule {rule.name} has an action {action.action} "
+                        "which needs inventory to be defined"
+                    )
