@@ -36,21 +36,27 @@ from drools import ruleset as lang
 
 from .collection import find_playbook, has_playbook, split_collection_name
 from .conf import settings
-from .exception import ShutdownException
+from .exception import (
+    PlaybookNotFoundException,
+    PlaybookStatusNotFoundException,
+    ShutdownException,
+)
 from .job_template_runner import job_template_runner
+from .messages import Shutdown
 from .util import get_horizontal_rule
 
 logger = logging.getLogger(__name__)
 
 tar = shutil.which("tar")
 
+KEY_EDA_VARS = "ansible_eda"
+
 
 async def none(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
@@ -70,10 +76,27 @@ async def none(
 
 
 async def debug(event_log, **kwargs):
-    print(get_horizontal_rule("="))
-    print("kwargs:")
-    pprint(kwargs)
-    print(get_horizontal_rule("="))
+    if "msg" in kwargs:
+        messages = kwargs.get("msg")
+        if not isinstance(messages, list):
+            messages = [messages]
+        for msg in messages:
+            print(msg)
+    elif "var" in kwargs:
+        key = kwargs.get("var")
+        try:
+            print(dpath.get(kwargs.get("variables"), key, separator="."))
+        except KeyError:
+            logger.error("Key %s not found in variable pool", key)
+            return
+    else:
+        print(get_horizontal_rule("="))
+        print("kwargs:")
+        pprint(kwargs)
+        print(get_horizontal_rule("="))
+        print("facts:")
+        pprint(lang.get_facts(kwargs["source_ruleset_name"]))
+        print(get_horizontal_rule("="))
     sys.stdout.flush()
     await event_log.put(
         dict(
@@ -91,28 +114,21 @@ async def debug(event_log, **kwargs):
 
 async def print_event(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
     ruleset: str,
     name: Optional[str] = None,
-    var_root: Union[str, Dict, None] = None,
     pretty: Optional[str] = None,
 ):
     print_fn: Callable = print
     if pretty:
         print_fn = pprint
 
-    if var_root:
-        update_variables(variables, var_root)
-
-    var_name = "event"
-    if "events" in variables:
-        var_name = "events"
+    var_name = "events" if "events" in variables else "event"
 
     print_fn(variables[var_name])
     sys.stdout.flush()
@@ -130,45 +146,11 @@ async def print_event(
     )
 
 
-async def echo(
-    event_log,
-    inventory: Dict,
-    hosts: List,
-    variables: Dict,
-    facts: Dict,
-    project_data_file: str,
-    source_ruleset_name: str,
-    source_rule_name: str,
-    ruleset: str,
-    name: Optional[str] = None,
-    var_root: Union[str, Dict, None] = None,
-    message: Optional[str] = None,
-):
-    if var_root:
-        update_variables(variables, var_root)
-
-    print(str(datetime.now()) + " : " + message)
-    sys.stdout.flush()
-    await event_log.put(
-        dict(
-            type="Action",
-            action="echo",
-            activation_id=settings.identifier,
-            ruleset=source_ruleset_name,
-            rule=source_rule_name,
-            playbook_name=name,
-            run_at=str(datetime.utcnow()),
-            matching_events=_get_events(variables),
-        )
-    )
-
-
 async def set_fact(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
@@ -194,10 +176,9 @@ async def set_fact(
 
 async def retract_fact(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
@@ -222,10 +203,9 @@ async def retract_fact(
 
 async def post_event(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
@@ -249,10 +229,9 @@ async def post_event(
 
 async def run_playbook(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
@@ -261,12 +240,12 @@ async def run_playbook(
     set_facts: Optional[bool] = None,
     post_events: Optional[bool] = None,
     verbosity: int = 0,
-    var_root: Union[str, Dict, None] = None,
     copy_files: Optional[bool] = False,
     json_mode: Optional[bool] = False,
     retries: Optional[int] = 0,
     retry: Optional[bool] = False,
     delay: Optional[int] = 0,
+    extra_vars: Optional[Dict] = None,
     **kwargs,
 ):
 
@@ -275,13 +254,14 @@ async def run_playbook(
         event_log,
         inventory,
         variables,
-        facts,
+        source_ruleset_name,
+        source_rule_name,
         name,
         "run_playbook",
-        var_root,
         copy_files,
         True,
         project_data_file,
+        extra_vars,
         **kwargs,
     )
 
@@ -346,10 +326,9 @@ async def run_playbook(
 
 async def run_module(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
@@ -358,27 +337,27 @@ async def run_module(
     set_facts: Optional[bool] = None,
     post_events: Optional[bool] = None,
     verbosity: int = 0,
-    var_root: Union[str, Dict, None] = None,
     copy_files: Optional[bool] = False,
     json_mode: Optional[bool] = False,
     module_args: Union[Dict, None] = None,
     retries: Optional[int] = 0,
     retry: Optional[bool] = False,
     delay: Optional[int] = 0,
+    extra_vars: Optional[Dict] = None,
     **kwargs,
 ):
-
     temp_dir, module_name = await pre_process_runner(
         event_log,
         inventory,
         variables,
-        facts,
+        source_ruleset_name,
+        source_rule_name,
         name,
         "run_module",
-        var_root,
         copy_files,
         False,
         project_data_file,
+        extra_vars,
         **kwargs,
     )
     job_id = str(uuid.uuid4())
@@ -458,6 +437,7 @@ async def call_runner(
 ):
 
     host_limit = ",".join(hosts)
+    shutdown = False
 
     loop = asyncio.get_running_loop()
 
@@ -484,30 +464,42 @@ async def call_runner(
                 val["run_at"] = event_data.get("created")
                 await event_log.put(val)
         except CancelledError:
-            pass
+            logger.info("Ansible Runner Queue task cancelled")
+
+    def cancel_callback():
+        return shutdown
 
     tasks = []
 
     tasks.append(asyncio.create_task(read_queue()))
 
     with concurrent.futures.ThreadPoolExecutor(max_workers=1) as task_pool:
-        await loop.run_in_executor(
-            task_pool,
-            partial(
-                ansible_runner.run,
-                private_data_dir=private_data_dir,
-                limit=host_limit,
-                verbosity=verbosity,
-                event_handler=event_callback,
-                json_mode=json_mode,
-                **runner_args,
-            ),
-        )
+        try:
+            await loop.run_in_executor(
+                task_pool,
+                partial(
+                    ansible_runner.run,
+                    private_data_dir=private_data_dir,
+                    limit=host_limit,
+                    verbosity=verbosity,
+                    event_handler=event_callback,
+                    cancel_callback=cancel_callback,
+                    json_mode=json_mode,
+                    **runner_args,
+                ),
+            )
+        except CancelledError:
+            logger.debug("Ansible Runner Thread Pool executor task cancelled")
+            shutdown = True
+            raise
+        finally:
+            # Cancel the queue reading task
+            for task in tasks:
+                if not task.done():
+                    logger.debug("Cancel Queue reading task")
+                    task.cancel()
 
-    # Cancel the queue reading task
-    for task in tasks:
-        task.cancel()
-    await asyncio.gather(*tasks)
+            await asyncio.gather(*tasks)
 
 
 async def untar_project(output_dir, project_data_file):
@@ -530,29 +522,25 @@ async def untar_project(output_dir, project_data_file):
 
 async def pre_process_runner(
     event_log,
-    inventory: Dict,
+    inventory: str,
     variables: Dict,
-    facts: Dict,
+    ruleset: str,
+    rulename: str,
     name: str,
     action: str,
-    var_root: Union[str, Dict, None] = None,
     copy_files: Optional[bool] = False,
     check_files: Optional[bool] = True,
     project_data_file: Optional[str] = None,
+    extra_vars: Optional[Dict] = None,
     **kwargs,
 ):
 
     private_data_dir = tempfile.mkdtemp(prefix=action)
     logger.debug("private data dir %s", private_data_dir)
-    logger.debug("variables %s", variables)
-    logger.debug("facts %s", facts)
 
-    variables["facts"] = facts
-    for k, v in kwargs.items():
-        variables[k] = v
-
-    if var_root:
-        update_variables(variables, var_root)
+    playbook_extra_vars = _collect_extra_vars(
+        variables, extra_vars, ruleset, rulename
+    )
 
     env_dir = os.path.join(private_data_dir, "env")
     inventory_dir = os.path.join(private_data_dir, "inventory")
@@ -562,10 +550,10 @@ async def pre_process_runner(
 
     os.mkdir(env_dir)
     with open(os.path.join(env_dir, "extravars"), "w") as f:
-        f.write(yaml.dump(variables))
+        f.write(yaml.dump(playbook_extra_vars))
     os.mkdir(inventory_dir)
     with open(os.path.join(inventory_dir, "hosts"), "w") as f:
-        f.write(yaml.dump(inventory))
+        f.write(inventory)
     os.mkdir(project_dir)
 
     logger.debug("project_data_file: %s", project_data_file)
@@ -591,9 +579,9 @@ async def pre_process_runner(
                 os.path.join(project_dir, name),
             )
         else:
-            logger.error(
-                "Could not find a playbook for %s from %s", name, os.getcwd()
-            )
+            msg = f"Could not find a playbook for {name} from {os.getcwd()}"
+            logger.error(msg)
+            raise PlaybookNotFoundException(msg)
 
     return (private_data_dir, playbook_name)
 
@@ -615,6 +603,12 @@ async def post_process_runner(
 
     rc = int(_get_latest_artifact(private_data_dir, "rc"))
     status = _get_latest_artifact(private_data_dir, "status")
+    logger.info("Playbook rc: %d, status: %s", rc, status)
+    if rc != 0:
+        error_message = _get_latest_artifact(private_data_dir, "stderr")
+        if not error_message:
+            error_message = _get_latest_artifact(private_data_dir, "stdout")
+        logger.error(error_message)
 
     result = dict(
         type="Action",
@@ -631,7 +625,7 @@ async def post_process_runner(
     )
     await event_log.put(result)
 
-    if set_facts or post_events:
+    if rc == 0 and (set_facts or post_events):
         logger.debug("set_facts")
         fact_folder = _get_latest_artifact(
             private_data_dir, "fact_cache", False
@@ -648,10 +642,9 @@ async def post_process_runner(
 
 async def run_job_template(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
@@ -662,7 +655,6 @@ async def run_job_template(
     set_facts: Optional[bool] = None,
     post_events: Optional[bool] = None,
     verbosity: int = 0,
-    var_root: Union[str, Dict, None] = None,
     copy_files: Optional[bool] = False,
     json_mode: Optional[bool] = False,
     retries: Optional[int] = 0,
@@ -681,10 +673,12 @@ async def run_job_template(
         job_args = {}
     job_args["limit"] = hosts_limit
 
-    job_args["extra_vars"] = job_args.get("extra_vars", {})
-    for key in ["fact", "facts", "event", "events"]:
-        if key in variables:
-            job_args["extra_vars"][key] = variables[key]
+    job_args["extra_vars"] = _collect_extra_vars(
+        variables,
+        job_args.get("extra_vars", {}),
+        source_ruleset_name,
+        source_rule_name,
+    )
 
     job_id = str(uuid.uuid4())
 
@@ -768,14 +762,16 @@ async def run_job_template(
 
 async def shutdown(
     event_log,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
     variables: Dict,
-    facts: Dict,
     project_data_file: str,
     source_ruleset_name: str,
     source_rule_name: str,
     ruleset: str,
+    delay: float = 60.0,
+    message: str = "Default shutdown message",
+    kind: str = "graceful",
 ):
     await event_log.put(
         dict(
@@ -786,9 +782,18 @@ async def shutdown(
             rule=source_rule_name,
             run_at=str(datetime.utcnow()),
             matching_events=_get_events(variables),
+            delay=delay,
+            message=message,
+            kind=kind,
         )
     )
-    raise ShutdownException()
+
+    print(
+        "Ruleset: %s rule: %s has initiated shutdown of type: %s. "
+        "Delay: %.3f seconds, Message: %s"
+        % (source_ruleset_name, source_rule_name, kind, delay, message)
+    )
+    raise ShutdownException(Shutdown(message=message, delay=delay, kind=kind))
 
 
 actions: Dict[str, Callable] = dict(
@@ -802,32 +807,14 @@ actions: Dict[str, Callable] = dict(
     run_module=run_module,
     run_job_template=run_job_template,
     shutdown=shutdown,
-    echo=echo,
 )
 
 
-def update_variables(variables: Dict, var_root: Union[str, Dict]):
-    var_roots = {var_root: var_root} if isinstance(var_root, str) else var_root
-    if "event" in variables:
-        for key, _new_key in var_roots.items():
-            new_value = dpath.get(
-                variables["event"], key, separator=".", default=None
-            )
-            if new_value:
-                variables["event"] = new_value
-                break
-    elif "events" in variables:
-        for _k, v in variables["events"].items():
-            for old_key, new_key in var_roots.items():
-                new_value = dpath.get(v, old_key, separator=".", default=None)
-                if new_value:
-                    variables["events"][new_key] = new_value
-                    break
-
-
 def _get_latest_artifact(data_dir: str, artifact: str, content: bool = True):
     files = glob.glob(os.path.join(data_dir, "artifacts", "*", artifact))
     files.sort(key=os.path.getmtime, reverse=True)
+    if not files:
+        raise PlaybookStatusNotFoundException(f"No {artifact} file found")
     if content:
         with open(files[0], "r") as f:
             content = f.read()
@@ -841,3 +828,16 @@ def _get_events(variables: Dict):
     elif "events" in variables:
         return variables["events"]
     return {}
+
+
+def _collect_extra_vars(
+    variables: Dict, user_extra_vars: Dict, ruleset: str, rule: str
+):
+    extra_vars = user_extra_vars.copy() if user_extra_vars else {}
+    eda_vars = dict(ruleset=ruleset, rule=rule)
+    if "events" in variables:
+        eda_vars["events"] = variables["events"]
+    if "event" in variables:
+        eda_vars["event"] = variables["event"]
+    extra_vars[KEY_EDA_VARS] = eda_vars
+    return extra_vars
