@@ -12,92 +12,74 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-"""
-Usage:
-    ansible-rulebook [options]
 
-Options:
-    -h, --help                  Show this page
-    -v, --vars=<v>              Variables file
-    -i, --inventory=<i>         Inventory
-    --rulebook=<r>              The rulebook file or rulebook from a collection
-    -S=<S>, --source_dir=<S>    Source dir
-    --vars=<v>                  A vars file
-    --env-vars=<e>              Comma separated list of variables to import
-                                from the environment
-    --redis_host_name=<h>       Redis host name
-    --redis_port=<p>            Redis port
-    --debug                     Show debug logging
-    --verbose                   Show verbose logging
-    --print-events              Print events after reading from source queue
-    --version                   Show the version and exit
-    --websocket-address=<w>     Connect the event log to a websocket
-    --id=<i>                    Identifier
-    --worker                    Enable worker mode
-    --project-tarball=<p>       Project tarball
-    --controller_url=<u>        Controller API base url, e.g. http://host1:8080
-    --controller_token=<t>      Controller API authentication token
-"""
 import argparse
 import asyncio
 import importlib.metadata
 import logging
 import os
 import sys
-from typing import List, NoReturn
+from typing import List
 
-from ansible_rulebook.job_template_runner import job_template_runner
+import ansible_rulebook.util as util
+from ansible_rulebook.messages import DEFAULT_SHUTDOWN_DELAY
 
-if os.environ.get("JAVA_HOME") is None:
-    print("JAVA_HOME is not set. " "Please install Java 11+ and set JAVA_HOME")
-    sys.exit(1)
+# ensure a valid JVM is available and configures JAVA_HOME if necessary
+# must be done before importing any other modules
+util.check_jvm()
+if not os.environ.get("JAVA_HOME"):
+    os.environ["JAVA_HOME"] = util.get_java_home()
 
-import ansible_rulebook
-from ansible_rulebook import app
-from ansible_rulebook.conf import settings
-from ansible_rulebook.util import get_java_version
+
+import ansible_rulebook  # noqa: E402
+from ansible_rulebook import app  # noqa: E402
+from ansible_rulebook.conf import settings  # noqa: E402
+from ansible_rulebook.job_template_runner import (  # noqa: E402
+    job_template_runner,
+)
+
+DEFAULT_VERBOSITY = 0
 
 logger = logging.getLogger(__name__)
 
 
 def get_parser() -> argparse.ArgumentParser:
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(
+        formatter_class=argparse.RawTextHelpFormatter
+    )
     parser.add_argument(
+        "-r",
         "--rulebook",
         help="The rulebook file or rulebook from a collection",
     )
     parser.add_argument(
+        "-e",
         "--vars",
         help="Variables file",
     )
     parser.add_argument(
+        "-E",
         "--env-vars",
         help=(
             "Comma separated list of variables to import from the environment"
         ),
     )
     parser.add_argument(
-        "--debug",
-        action="store_true",
-        help="Show debug logging, written to stdout",
-    )
-    parser.add_argument(
+        "-v",
         "--verbose",
-        action="store_true",
-        help="Show verbose logging, written to stdout",
+        dest="verbosity",
+        default=DEFAULT_VERBOSITY,
+        action="count",
+        help="Causes ansible-rulebook to print more debug messages. "
+        "Adding multiple -v will increase the verbosity, "
+        "the default value is 0. The maximum value is 2. "
+        "Events debugging might require -vv.",
     )
     parser.add_argument(
         "--version",
-        action="store_true",
+        action="version",
         help="Show the version and exit",
-    )
-    parser.add_argument(
-        "--redis-host-name",
-        help="Redis host name",
-    )
-    parser.add_argument(
-        "--redis-port",
-        help="Redis port",
+        version=get_version(),
     )
     parser.add_argument(
         "-S",
@@ -110,48 +92,64 @@ def get_parser() -> argparse.ArgumentParser:
         help="Inventory",
     )
     parser.add_argument(
+        "-W",
         "--websocket-address",
         help="Connect the event log to a websocket",
     )
     parser.add_argument("--id", help="Identifier", type=int)
     parser.add_argument(
+        "-w",
         "--worker",
         action="store_true",
         help="Enable worker mode",
     )
     parser.add_argument(
+        "-T",
         "--project-tarball",
         help="A tarball of the project",
     )
     parser.add_argument(
         "--controller-url",
-        help="Controller API base url, e.g. http://host1:8080",
+        help="Controller API base url, e.g. https://host1:8080",
     )
     parser.add_argument(
         "--controller-token",
         help="Controller API authentication token",
     )
+    parser.add_argument(
+        "--controller-ssl-verify",
+        help="How to verify SSL when connecting to the "
+        "controller, yes|no|<path to a CA bundle>, "
+        "default to yes for https connection",
+    )
     parser.add_argument(
         "--print-events",
         action="store_true",
-        help="Print events to stdout, disabled if used with --debug",
+        help="Print events to stdout, redundant and disabled with -vv",
+    )
+    parser.add_argument(
+        "--shutdown-delay",
+        default=os.environ.get("EDA_SHUTDOWN_DELAY", DEFAULT_SHUTDOWN_DELAY),
+        type=float,
+        help="Maximum number of seconds to wait after issuing a "
+        "graceful shutdown, default: 60. The process will shutdown if "
+        "all actions complete before this time period",
     )
     return parser
 
 
-def show_version() -> NoReturn:
-    java_home = f"{os.environ.get('JAVA_HOME', 'JAVA_HOME not present')}"
-
+def get_version() -> str:
+    java_home = util.get_java_home()
+    java_version = util.get_java_version()
     result = [
         f"{ansible_rulebook.__version__}",
         f"  Executable location = {sys.argv[0]}",
         f"  Drools_jpy version = {importlib.metadata.version('drools_jpy')}",
         f"  Java home = {java_home}",
-        f"  Java version = {get_java_version()}",
+        f"  Java version = {java_version}",
         f"  Python version = {''.join(sys.version.splitlines())}",
     ]
-    print("\n".join(result))
-    sys.exit(0)
+    return "\n".join(result)
 
 
 def setup_logging(args: argparse.Namespace) -> None:
@@ -159,14 +157,16 @@ def setup_logging(args: argparse.Namespace) -> None:
     stream = sys.stderr
     level = logging.WARNING
 
-    if args.debug:
+    if args.verbosity >= 2:
         level = logging.DEBUG
         stream = sys.stdout
-    elif args.verbose:
+        args.print_events = False
+    elif args.verbosity == 1:
         level = logging.INFO
         stream = sys.stdout
 
     logging.basicConfig(stream=stream, level=level, format=LOG_FORMAT)
+    logging.getLogger("drools.").setLevel(level)
 
 
 def main(args: List[str] = None) -> int:
@@ -177,19 +177,12 @@ def main(args: List[str] = None) -> int:
         parser.print_help()
         sys.exit(0)
 
-    if args.version:
-        show_version()
-
-    if args.rulebook and not args.inventory:
-        print("Error: inventory is required")
-        return 1
-
-    if args.debug:
-        args.print_events = False
     if args.controller_url:
         if args.controller_token:
             job_template_runner.host = args.controller_url
             job_template_runner.token = args.controller_token
+            if args.controller_ssl_verify:
+                job_template_runner.verify_ssl = args.controller_ssl_verify
         else:
             print("Error: controller_token is required")
             return 1
