@@ -13,13 +13,13 @@
 #  limitations under the License.
 
 import logging
-import sys
 
 from pyparsing import (
     Combine,
     Group,
     Literal,
     OpAssoc,
+    Optional,
     ParseException,
     ParserElement,
     QuotedString,
@@ -31,28 +31,62 @@ from pyparsing import (
     pyparsing_common,
 )
 
+from ansible_rulebook.exception import (
+    ConditionParsingException,
+    SelectattrOperatorException,
+    SelectOperatorException,
+)
+
 ParserElement.enable_packrat()
 
 from ansible_rulebook.condition_types import (  # noqa: E402
     Boolean,
     Condition,
-    ExistsExpression,
-    Float,
     Identifier,
-    Integer,
+    KeywordValue,
     NegateExpression,
+    Null,
     OperatorExpression,
+    SearchType,
+    SelectattrType,
+    SelectType,
     String,
+    to_condition_type,
 )
 
+VALID_SELECT_ATTR_OPERATORS = [
+    "==",
+    "!=",
+    ">",
+    ">=",
+    "<",
+    "<=",
+    "regex",
+    "search",
+    "match",
+    "in",
+    "not in",
+    "contains",
+    "not contains",
+]
+
+VALID_SELECT_OPERATORS = [
+    "==",
+    "!=",
+    ">",
+    ">=",
+    "<",
+    "<=",
+    "regex",
+    "search",
+    "match",
+]
+SUPPORTED_SEARCH_KINDS = ("match", "regex", "search")
+
 logger = logging.getLogger(__name__)
 
-integer = pyparsing_common.signed_integer.copy().add_parse_action(
-    lambda toks: Integer(toks[0])
-)
-
-float_t = pyparsing_common.real.copy().add_parse_action(
-    lambda toks: Float(toks[0])
+number_t = pyparsing_common.number.copy().add_parse_action(
+    lambda toks: to_condition_type(toks[0])
 )
 
 ident = pyparsing_common.identifier
@@ -69,6 +103,7 @@ boolean = (
     .add_parse_action(lambda toks: Boolean(toks[0].lower()))
 )
 
+null_t = Literal("null").copy().add_parse_action(lambda toks: Null())
 
 string1 = (
     QuotedString("'").copy().add_parse_action(lambda toks: String(toks[0]))
@@ -77,11 +112,34 @@ string2 = (
     QuotedString('"').copy().add_parse_action(lambda toks: String(toks[0]))
 )
 
+allowed_values = number_t | boolean | null_t | string1 | string2
+key_value = ident + Suppress("=") + allowed_values
+string_search_t = (
+    one_of("regex match search")
+    + Suppress("(")
+    + Group(Optional(delimitedList(string1 | string2 | key_value)))
+    + Suppress(")")
+)
+
 delim_value = Group(
-    delimitedList(float_t | integer | ident | string1 | string2)
+    delimitedList(number_t | null_t | boolean | ident | string1 | string2)
 )
 list_values = Suppress("[") + delim_value + Suppress("]")
 
+selectattr_t = (
+    Literal("selectattr")
+    + Suppress("(")
+    + Group(delimitedList(allowed_values | ident | list_values))
+    + Suppress(")")
+)
+
+select_t = (
+    Literal("select")
+    + Suppress("(")
+    + Group(delimitedList(allowed_values | ident | list_values))
+    + Suppress(")")
+)
+
 
 def as_list(var):
     if hasattr(var.__class__, "as_list"):
@@ -89,14 +147,60 @@ def as_list(var):
     return var
 
 
+def SelectattrTypeFactory(tokens):
+    if tokens[1].value not in VALID_SELECT_ATTR_OPERATORS:
+        raise SelectattrOperatorException(
+            f"Operator {tokens[1]} is not supported"
+        )
+
+    return SelectattrType(tokens[0], tokens[1], as_list(tokens[2]))
+
+
+def SelectTypeFactory(tokens):
+    if tokens[0].value not in VALID_SELECT_OPERATORS:
+        raise SelectOperatorException(f"Operator {tokens[0]} is not supported")
+
+    return SelectType(tokens[0], as_list(tokens[1]))
+
+
+def SearchTypeFactory(kind, tokens):
+    options = []
+    if len(tokens) > 1:
+        for i in range(1, len(tokens), 2):
+            options.append(KeywordValue(String(tokens[i]), tokens[i + 1]))
+
+    return SearchType(String(kind), tokens[0], options)
+
+
 def OperatorExpressionFactory(tokens):
     return_value = None
     while tokens:
         if return_value is None:
-            return_value = OperatorExpression(
-                as_list(tokens[0]), tokens[1], as_list(tokens[2])
-            )
-            tokens = tokens[3:]
+            if (tokens[1] == "is" or tokens[1] == "is not") and (
+                tokens[2] in SUPPORTED_SEARCH_KINDS
+            ):
+                search_type = SearchTypeFactory(tokens[2], tokens[3])
+                return_value = OperatorExpression(
+                    tokens[0], tokens[1], search_type
+                )
+                tokens = tokens[4:]
+            elif tokens[2] == "selectattr":
+                select_attr_type = SelectattrTypeFactory(tokens[3])
+                return_value = OperatorExpression(
+                    tokens[0], tokens[1], select_attr_type
+                )
+                tokens = tokens[4:]
+            elif tokens[2] == "select":
+                select_type = SelectTypeFactory(tokens[3])
+                return_value = OperatorExpression(
+                    tokens[0], tokens[1], select_type
+                )
+                tokens = tokens[4:]
+            else:
+                return_value = OperatorExpression(
+                    as_list(tokens[0]), tokens[1], as_list(tokens[2])
+                )
+                tokens = tokens[3:]
         else:
             return_value = OperatorExpression(
                 return_value, tokens[0], tokens[1]
@@ -106,13 +210,20 @@ def OperatorExpressionFactory(tokens):
 
 
 all_terms = (
-    list_values | float_t | integer | boolean | varname | string1 | string2
+    selectattr_t
+    | select_t
+    | string_search_t
+    | list_values
+    | number_t
+    | null_t
+    | boolean
+    | varname
+    | string1
+    | string2
 )
 condition = infix_notation(
     all_terms,
     [
-        ("+", 1, OpAssoc.RIGHT, lambda toks: ExistsExpression(*toks[0])),
-        ("not", 1, OpAssoc.RIGHT, lambda toks: NegateExpression(*toks[0])),
         (
             one_of("* /"),
             2,
@@ -156,13 +267,7 @@ condition = infix_notation(
             lambda toks: OperatorExpressionFactory(toks[0]),
         ),
         (
-            "is not",
-            2,
-            OpAssoc.LEFT,
-            lambda toks: OperatorExpressionFactory(toks[0]),
-        ),
-        (
-            "is",
+            one_of(strs=["is not", "is"]),
             2,
             OpAssoc.LEFT,
             lambda toks: OperatorExpressionFactory(toks[0]),
@@ -177,6 +282,7 @@ condition = infix_notation(
             OpAssoc.LEFT,
             lambda toks: OperatorExpressionFactory(toks[0]),
         ),
+        ("not", 1, OpAssoc.RIGHT, lambda toks: NegateExpression(*toks[0])),
         (
             one_of(["and", "or"]),
             2,
@@ -197,6 +303,6 @@ def parse_condition(condition_string: str) -> Condition:
     try:
         return condition.parseString(condition_string, parse_all=True)[0]
     except ParseException as pe:
-        print(pe.explain(depth=0), file=sys.stderr)
-        logger.error(pe.explain(depth=0))
-        raise
+        msg = f"Error parsing: {condition_string}. {pe}"
+        logger.debug(pe.explain(depth=0))
+        raise ConditionParsingException(msg)
