@@ -21,15 +21,24 @@ from ansible_rulebook.condition_types import (
     Boolean,
     Condition,
     ConditionTypes,
-    ExistsExpression,
     Float,
     Identifier,
     Integer,
+    KeywordValue,
     NegateExpression,
+    Null,
     OperatorExpression,
+    SearchType,
+    SelectattrType,
+    SelectType,
     String,
+    to_condition_type,
+)
+from ansible_rulebook.exception import (
+    InvalidAssignmentException,
+    InvalidIdentifierException,
+    VarsKeyMissingException,
 )
-from ansible_rulebook.exception import VarsKeyMissingException
 from ansible_rulebook.rule_types import (
     Action,
     Condition as RuleCondition,
@@ -37,6 +46,7 @@ from ansible_rulebook.rule_types import (
     EventSourceFilter,
     Rule,
     RuleSet,
+    Throttle,
 )
 from ansible_rulebook.util import substitute_variables
 
@@ -84,7 +94,9 @@ def visit_condition(parsed_condition: ConditionTypes, variables: Dict):
             key = parsed_condition.value[5:]
             try:
                 return visit_condition(
-                    convert_to_type(dpath.get(variables, key, separator=".")),
+                    to_condition_type(
+                        dpath.get(variables, key, separator=".")
+                    ),
                     variables,
                 )
             except KeyError:
@@ -92,16 +104,52 @@ def visit_condition(parsed_condition: ConditionTypes, variables: Dict):
                     f"vars does not contain key: {key}"
                 )
         else:
-            raise Exception(f"Unhandled identifier {parsed_condition.value}")
+            msg = (
+                f"Invalid identifier : {parsed_condition.value} "
+                + "Should start with event., events.,fact., facts. or vars."
+            )
+            raise InvalidIdentifierException(msg)
+
     elif isinstance(parsed_condition, String):
         return {
             "String": substitute_variables(parsed_condition.value, variables)
         }
+    elif isinstance(parsed_condition, Null):
+        return {"NullType": None}
     elif isinstance(parsed_condition, Integer):
         return {"Integer": parsed_condition.value}
     elif isinstance(parsed_condition, Float):
         return {"Float": parsed_condition.value}
+    elif isinstance(parsed_condition, SearchType):
+        data = dict(
+            kind=visit_condition(parsed_condition.kind, variables),
+            pattern=visit_condition(parsed_condition.pattern, variables),
+        )
+        if parsed_condition.options:
+            data["options"] = [
+                visit_condition(v, variables) for v in parsed_condition.options
+            ]
+        return {"SearchType": data}
+    elif isinstance(parsed_condition, SelectattrType):
+        return dict(
+            key=visit_condition(parsed_condition.key, variables),
+            operator=visit_condition(parsed_condition.operator, variables),
+            value=visit_condition(parsed_condition.value, variables),
+        )
+    elif isinstance(parsed_condition, SelectType):
+        return dict(
+            operator=visit_condition(parsed_condition.operator, variables),
+            value=visit_condition(parsed_condition.value, variables),
+        )
+    elif isinstance(parsed_condition, KeywordValue):
+        return dict(
+            name=visit_condition(parsed_condition.name, variables),
+            value=visit_condition(parsed_condition.value, variables),
+        )
     elif isinstance(parsed_condition, OperatorExpression):
+        if parsed_condition.operator == "<<":
+            validate_assignment_expression(parsed_condition.left.value)
+
         if parsed_condition.operator in OPERATOR_MNEMONIC:
             return create_binary_node(
                 OPERATOR_MNEMONIC[parsed_condition.operator],
@@ -116,6 +164,18 @@ def visit_condition(parsed_condition: ConditionTypes, variables: Dict):
                             parsed_condition.left, variables
                         )
                     }
+            elif isinstance(parsed_condition.right, SearchType):
+                return create_binary_node(
+                    "SearchMatchesExpression", parsed_condition, variables
+                )
+            elif isinstance(parsed_condition.right, SelectattrType):
+                return create_binary_node(
+                    "SelectAttrExpression", parsed_condition, variables
+                )
+            elif isinstance(parsed_condition.right, SelectType):
+                return create_binary_node(
+                    "SelectExpression", parsed_condition, variables
+                )
         elif parsed_condition.operator == "is not":
             if isinstance(parsed_condition.right, Identifier):
                 if parsed_condition.right.value == "defined":
@@ -124,13 +184,20 @@ def visit_condition(parsed_condition: ConditionTypes, variables: Dict):
                             parsed_condition.left, variables
                         )
                     }
+            elif isinstance(parsed_condition.right, SearchType):
+                return create_binary_node(
+                    "SearchNotMatchesExpression", parsed_condition, variables
+                )
+            elif isinstance(parsed_condition.right, SelectattrType):
+                return create_binary_node(
+                    "SelectAttrNotExpression", parsed_condition, variables
+                )
+            elif isinstance(parsed_condition.right, SelectType):
+                return create_binary_node(
+                    "SelectNotExpression", parsed_condition, variables
+                )
         else:
             raise Exception(f"Unhandled token {parsed_condition}")
-    elif isinstance(parsed_condition, ExistsExpression):
-        if parsed_condition.operator == "+":
-            raise Exception("Please use 'is defined' instead of +")
-
-        return visit_condition(parsed_condition.value, variables).__pos__()
     elif isinstance(parsed_condition, NegateExpression):
         return {
             "NegateExpression": visit_condition(
@@ -141,21 +208,6 @@ def visit_condition(parsed_condition: ConditionTypes, variables: Dict):
         raise Exception(f"Unhandled token {parsed_condition}")
 
 
-def convert_to_type(value):
-    if isinstance(value, int):
-        return Integer(value)
-    elif isinstance(value, bool):
-        return Boolean(value)
-    elif isinstance(value, str):
-        return String(value)
-    elif isinstance(value, float):
-        return Float(value)
-    elif isinstance(value, list):
-        return [convert_to_type(v) for v in value]
-    else:
-        raise Exception(f"Invalid type for {value}")
-
-
 def create_binary_node(name, parsed_condition, variables):
     return {
         name: {
@@ -166,15 +218,22 @@ def create_binary_node(name, parsed_condition, variables):
 
 
 def visit_rule(parsed_rule: Rule, variables: Dict):
-    return {
-        "Rule": {
-            "name": parsed_rule.name,
-            "condition": generate_condition(parsed_rule.condition, variables),
-            "action": visit_action(parsed_rule.action, variables),
-            "enabled": parsed_rule.enabled,
-        }
+    data = {
+        "name": parsed_rule.name,
+        "condition": generate_condition(parsed_rule.condition, variables),
+        "actions": visit_actions(parsed_rule.actions, variables),
+        "enabled": parsed_rule.enabled,
     }
 
+    if parsed_rule.throttle:
+        data.update(visit_throttle(parsed_rule.throttle, variables))
+
+    return {"Rule": data}
+
+
+def visit_actions(actions: List[Action], variables: Dict):
+    return [visit_action(a, variables) for a in actions]
+
 
 def visit_action(parsed_action: Action, variables: Dict):
     return {
@@ -185,6 +244,16 @@ def visit_action(parsed_action: Action, variables: Dict):
     }
 
 
+def visit_throttle(parsed_throttle: Throttle, variables: Dict):
+    throttle = {"group_by_attributes": parsed_throttle.group_by_attributes}
+    if parsed_throttle.once_within:
+        throttle["once_within"] = parsed_throttle.once_within
+    elif parsed_throttle.once_after:
+        throttle["once_after"] = parsed_throttle.once_after
+
+    return {"throttle": throttle}
+
+
 def visit_source(parsed_source: EventSource, variables: Dict):
     return {
         "EventSource": {
@@ -213,11 +282,18 @@ def generate_condition(ansible_condition: RuleCondition, variables: Dict):
     """Generate the condition AST."""
     condition = visit_condition(ansible_condition.value, variables)
     if ansible_condition.when == "any":
-        return {"AnyCondition": condition}
+        data = {"AnyCondition": condition}
     elif ansible_condition.when == "all":
-        return {"AllCondition": condition}
+        data = {"AllCondition": condition}
+    elif ansible_condition.when == "not_all":
+        data = {"NotAllCondition": condition}
     else:
-        return {"AllCondition": condition}
+        data = {"AllCondition": condition}
+
+    if ansible_condition.timeout:
+        data["timeout"] = ansible_condition.timeout
+
+    return data
 
 
 def visit_ruleset(ruleset: RuleSet, variables: Dict):
@@ -237,3 +313,22 @@ def visit_ruleset(ruleset: RuleSet, variables: Dict):
 def generate_dict_rulesets(ruleset: List[RuleSet], variables: Dict):
     """Generate JSON compatible rulesets."""
     return [visit_ruleset(ruleset, variables) for ruleset in ruleset]
+
+
+def validate_assignment_expression(value):
+    tokens = value.split(".")
+    if len(tokens) != 2:
+        msg = (
+            f"Assignment variable: {value} is invalid."
+            + "Valid values start with events or facts. e.g events.var1 "
+            + "or facts.var1 "
+            + "Where var1 can only contain alpha numeric and _ charachters"
+        )
+        raise InvalidAssignmentException(msg)
+
+    if tokens[0] not in ["events", "facts"]:
+        msg = (
+            "Only events and facts can be used in assignment. "
+            + f"{value} is invalid."
+        )
+        raise InvalidAssignmentException(msg)
