@@ -15,13 +15,14 @@
 import asyncio
 import json
 import logging
-from typing import Callable, Dict, List
+from typing import Any, Callable, Dict, List
 
 from drools.rule import Rule as DroolsRule
 from drools.ruleset import Ruleset as DroolsRuleset
 
 from ansible_rulebook.json_generator import visit_ruleset
 from ansible_rulebook.rule_types import (
+    Action,
     ActionContext,
     EngineRuleSetQueuePlan,
     Plan,
@@ -34,26 +35,22 @@ logger = logging.getLogger(__name__)
 def add_to_plan(
     ruleset: str,
     rule: str,
-    action: str,
-    action_args: Dict,
+    actions: List[Action],
     variables: Dict,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
-    facts: Dict,
     plan: Plan,
-    c,
+    rule_engine_results: Any,
 ) -> None:
     plan.queue.put_nowait(
         ActionContext(
             ruleset,
             rule,
-            action,
-            action_args,
+            actions,
             variables,
             inventory,
             hosts,
-            facts,
-            c,
+            rule_engine_results,
         )
     )
 
@@ -62,24 +59,21 @@ def make_fn(
     ruleset,
     ansible_rule,
     variables: Dict,
-    inventory: Dict,
+    inventory: str,
     hosts: List,
-    facts: Dict,
     plan: Plan,
 ) -> Callable:
-    def fn(c):
+    def fn(rule_engine_results):
         logger.info("calling %s", ansible_rule.name)
         add_to_plan(
             ruleset,
             ansible_rule.name,
-            ansible_rule.action.action,
-            ansible_rule.action.action_args,
+            ansible_rule.actions,
             variables,
             inventory,
             hosts,
-            facts,
             plan,
-            c,
+            rule_engine_results,
         )
 
     return fn
@@ -88,7 +82,7 @@ def make_fn(
 def generate_rulesets(
     ruleset_queues: List[RuleSetQueue],
     variables: Dict,
-    inventory: Dict,
+    inventory: str,
 ) -> List[EngineRuleSetQueuePlan]:
 
     rulesets = []
@@ -108,7 +102,6 @@ def generate_rulesets(
                     variables,
                     inventory,
                     ansible_ruleset.hosts,
-                    {},
                     plan,
                 )
                 drools_ruleset.add_rule(
