@@ -12,15 +12,21 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Optional
 
 import ansible_rulebook.rule_types as rt
 from ansible_rulebook.condition_parser import (
     parse_condition as parse_condition_value,
 )
 from ansible_rulebook.job_template_runner import job_template_runner
+from ansible_rulebook.util import substitute_variables
 
-from .exception import RulenameDuplicateException, RulenameEmptyException
+from .exception import (
+    RulenameDuplicateException,
+    RulenameEmptyException,
+    RulesetNameDuplicateException,
+    RulesetNameEmptyException,
+)
 
 
 def parse_hosts(hosts):
@@ -32,15 +38,38 @@ def parse_hosts(hosts):
         raise Exception(f"Unsupported hosts value {hosts}")
 
 
-def parse_rule_sets(rule_sets: Dict) -> List[rt.RuleSet]:
+def parse_rule_sets(
+    rule_sets: Dict, variables: Optional[Dict] = None
+) -> List[rt.RuleSet]:
     rule_set_list = []
+    ruleset_names = []
     for rule_set in rule_sets:
+        name = rule_set.get("name")
+        if name is None:
+            raise RulesetNameEmptyException("Ruleset name not provided")
+
+        name = name.strip()
+        if name == "":
+            raise RulesetNameEmptyException(
+                "Ruleset name cannot be an empty string"
+            )
+
+        if name in ruleset_names:
+            raise RulesetNameDuplicateException(
+                f"Ruleset with name: {name} defined multiple times"
+            )
+
+        ruleset_names.append(name)
+
+        if variables is None:
+            variables = {}
+
         rule_set_list.append(
             rt.RuleSet(
-                name=rule_set["name"],
+                name=name,
                 hosts=parse_hosts(rule_set["hosts"]),
                 sources=parse_event_sources(rule_set["sources"]),
-                rules=parse_rules(rule_set.get("rules", {})),
+                rules=parse_rules(rule_set.get("rules", {}), variables),
                 gather_facts=rule_set.get("gather_facts", False),
             )
         )
@@ -79,14 +108,17 @@ def parse_source_filter(source_filter: Dict) -> rt.EventSourceFilter:
     return rt.EventSourceFilter(source_filter_name, source_filter_args)
 
 
-def parse_rules(rules: Dict) -> List[rt.Rule]:
+def parse_rules(rules: Dict, variables: Dict) -> List[rt.Rule]:
     rule_list = []
     rule_names = []
+    if variables is None:
+        variables = {}
     for rule in rules:
         name = rule.get("name")
         if name is None:
             raise RulenameEmptyException("Rule name not provided")
 
+        name = substitute_variables(name, variables)
         if name == "":
             raise RulenameEmptyException("Rule name cannot be an empty string")
 
@@ -96,18 +128,39 @@ def parse_rules(rules: Dict) -> List[rt.Rule]:
             )
 
         rule_names.append(name)
-        rule_list.append(
-            rt.Rule(
-                name=name,
-                condition=parse_condition(rule["condition"]),
-                action=parse_action(rule["action"]),
-                enabled=rule.get("enabled", True),
+        if "throttle" in rule:
+            throttle = rt.Throttle(
+                once_within=rule["throttle"].get("once_within", None),
+                once_after=rule["throttle"].get("once_after", None),
+                group_by_attributes=rule["throttle"]["group_by_attributes"],
             )
+        else:
+            throttle = None
+
+        rule = rt.Rule(
+            name=name,
+            condition=parse_condition(rule["condition"]),
+            actions=parse_actions(rule),
+            enabled=rule.get("enabled", True),
+            throttle=throttle,
         )
+        if rule.enabled:
+            rule_list.append(rule)
 
     return rule_list
 
 
+def parse_actions(rule: Dict) -> List[rt.Action]:
+    actions = []
+    if "actions" in rule:
+        for action in rule["actions"]:
+            actions.append(parse_action(action))
+    elif "action" in rule:
+        actions.append(parse_action(rule["action"]))
+
+    return actions
+
+
 def parse_action(action: Dict) -> rt.Action:
     action_name = list(action.keys())[0]
     if action_name == "run_job_template":
@@ -125,15 +178,18 @@ def parse_condition(condition: Any) -> rt.Condition:
     if isinstance(condition, str):
         return rt.Condition("all", [parse_condition_value(condition)])
     elif isinstance(condition, dict):
+        timeout = condition.pop("timeout", None)
         keys = list(condition.keys())
-        if len(condition) == 1 and keys[0] in ["any", "all"]:
+        if len(condition) == 1 and keys[0] in ["any", "all", "not_all"]:
             when = keys[0]
             return rt.Condition(
-                when, [parse_condition_value(c) for c in condition[when]]
+                when,
+                [parse_condition_value(c) for c in condition[when]],
+                timeout,
             )
         else:
             raise Exception(
-                f"Condition should have one of any or all: {condition}"
+                f"Condition should have one of any, all, not_all: {condition}"
             )
 
     else:
