@@ -14,16 +14,21 @@
 
 import glob
 import json
+import logging
 import os
 import shutil
 import subprocess
+import sys
 import tempfile
-from pprint import pprint
+import typing
 from typing import Any, Dict, List, Union
 
 import ansible_runner
 import jinja2
-import yaml
+from jinja2.nativetypes import NativeTemplate
+from packaging import version
+
+logger = logging.getLogger(__name__)
 
 
 def get_horizontal_rule(character):
@@ -34,9 +39,12 @@ def get_horizontal_rule(character):
 
 
 def render_string(value: str, context: Dict) -> str:
-    return jinja2.Template(value, undefined=jinja2.StrictUndefined).render(
-        context
-    )
+    if "{{" in value and "}}" in value:
+        return NativeTemplate(value, undefined=jinja2.StrictUndefined).render(
+            context
+        )
+
+    return value
 
 
 def render_string_or_return_value(value: Any, context: Dict) -> Any:
@@ -53,12 +61,12 @@ def substitute_variables(
     elif isinstance(value, list):
         new_value = []
         for item in value:
-            new_value.append(render_string_or_return_value(item, context))
+            new_value.append(substitute_variables(item, context))
         return new_value
     elif isinstance(value, dict):
         new_value = value.copy()
         for key, subvalue in new_value.items():
-            new_value[key] = render_string_or_return_value(subvalue, context)
+            new_value[key] = substitute_variables(subvalue, context)
         return new_value
     else:
         return value
@@ -67,33 +75,11 @@ def substitute_variables(
 def load_inventory(inventory_file: str) -> Any:
 
     with open(inventory_file) as f:
-        inventory_data = yaml.safe_load(f.read())
+        inventory_data = f.read()
     return inventory_data
 
 
-def json_count(data):
-    s = 0
-    q = []
-    q.append(data)
-    while q:
-        o = q.pop()
-        if isinstance(o, dict):
-            s += len(o)
-            if len(o) > 255:
-                pprint(data)
-                raise Exception(
-                    f"Only 255 values supported per dictionary found {len(o)}"
-                )
-            if s > 255:
-                pprint(data)
-                raise Exception(
-                    f"Only 255 values supported per dictionary found {s}"
-                )
-            for i in o.values():
-                q.append(i)
-
-
-def collect_ansible_facts(inventory: Dict) -> List[Dict]:
+def collect_ansible_facts(inventory: str) -> List[Dict]:
     hosts_facts = []
     with tempfile.TemporaryDirectory(
         prefix="gather_facts"
@@ -102,7 +88,7 @@ def collect_ansible_facts(inventory: Dict) -> List[Dict]:
         with open(
             os.path.join(private_data_dir, "inventory", "hosts"), "w"
         ) as f:
-            f.write(yaml.dump(inventory))
+            f.write(inventory)
 
         r = ansible_runner.run(
             private_data_dir=private_data_dir,
@@ -128,16 +114,83 @@ def collect_ansible_facts(inventory: Dict) -> List[Dict]:
     return hosts_facts
 
 
-def get_java_version() -> str:
+def run_java_settings(exec_path: str) -> subprocess.CompletedProcess:
+    return subprocess.run(
+        [exec_path, "-XshowSettings:properties", "-version"],
+        check=True,
+        text=True,
+        stdout=subprocess.DEVNULL,
+        stderr=subprocess.PIPE,
+    )
+
+
+def get_java_home() -> typing.Optional[str]:
+    """
+    Get the java home path. It tries to get the path
+    from the default java installation if JAVA_HOME is not set.
+    If is not possible to find the java home path, it returns None.
+    """
+
+    # if java_home is set, return it
+    java_home = os.environ.get("JAVA_HOME", None)
+    if java_home:
+        return java_home
+
+    # if default java executable is not found, return None
     exec_path = shutil.which("java")
     if not exec_path:
-        return "Java executable not found."
-    try:
-        result = subprocess.run(
-            [exec_path, "--version"], capture_output=True, check=True
-        )
-    except subprocess.CalledProcessError as exc:
-        print(exc)
-        return "Java error"
+        return None
 
-    return result.stdout.splitlines()[0].decode()
+    # try to get the java home path from the default java executable
+    try:
+        result = run_java_settings(exec_path)
+    except subprocess.CalledProcessError:
+        return None
+
+    for line in result.stderr.splitlines():
+        if "java.home" in line:
+            return line.split("=")[-1].strip()
+
+    return None
+
+
+def get_java_version() -> str:
+    java_home = get_java_home()
+
+    if not java_home:
+        return "Java executable not found."
+
+    exec_path = f"{java_home}/bin/java"
+    try:
+        result = run_java_settings(exec_path)
+    except subprocess.CalledProcessError as exc:
+        logger.error("java executable failed: %s", exc)
+        return "Java error"
+    for line in result.stderr.splitlines():
+        if "java.version" in line:
+            return line.split("=")[-1].strip()
+
+    return "Java version not found."
+
+
+def check_jvm():
+    """
+    Ensures that a valid JVM is properly installed
+    """
+    java_home = get_java_home()
+    if not java_home:
+        print(
+            "Java executable or JAVA_HOME environment variable not found."
+            "Please install a valid JVM.",
+            file=sys.stderr,
+        )
+        sys.exit(1)
+
+    java_version = get_java_version()
+    if version.parse(java_version) < version.parse("17"):
+        print(
+            "The minimum supported Java version is 17. "
+            f"Found version: {java_version}",
+            file=sys.stderr,
+        )
+        sys.exit(1)
