@@ -8,8 +8,8 @@ if sys.version_info >= (3, 9):
 else:
     import importlib_resources as resources
 
-from jsonschema import validate
-from jsonschema.exceptions import ValidationError
+import jsonschema
+from jsonschema.exceptions import SchemaError, ValidationError
 
 DEFAULT_RULEBOOK_SCHEMA = "ruleset_schema"
 logger = logging.getLogger(__name__)
@@ -27,13 +27,22 @@ class Validate:
             f"./schema/{DEFAULT_RULEBOOK_SCHEMA}.json"
         )
         data = path.read_text(encoding="utf-8")
-        cls.schema = json.loads(data)
+        try:
+            cls.schema = json.loads(data)
+            validator = jsonschema.validators.validator_for(cls.schema)
+            validator.check_schema(cls.schema)
+        except json.JSONDecodeError:
+            logger.exception("Can not deserialize JSON schema")
+            raise
+        except SchemaError:
+            logger.exception("Incorrect JSON schema")
+            raise
         return cls.schema
 
     @classmethod
     def rulebook(cls, instance: List[Dict]) -> None:
         try:
-            validate(instance=instance, schema=cls._get_schema())
+            jsonschema.validate(instance=instance, schema=cls._get_schema())
         except ValidationError:
             logger.exception("Rulebook failed validation.")
             raise
