@@ -1,5 +1,19 @@
 #!/usr/bin/env python3
 
+#  Copyright 2022 Red Hat, Inc.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+
 import os
 import traceback
 
@@ -12,7 +26,9 @@ HERE = os.path.dirname(os.path.abspath(__file__))
 
 
 def main():
-    for root, _, files in os.walk(os.path.join(HERE, "rules")):
+    rules = list(os.walk(os.path.join(HERE, "rules")))
+    rules.extend(list(os.walk(os.path.join(HERE, "examples"))))
+    for root, _, files in rules:
         for file in files:
             if file.endswith(".yml"):
                 print(os.path.join(root, file))
