@@ -18,6 +18,13 @@ import pytest
 import yaml
 
 from ansible_rulebook.condition_parser import parse_condition
+from ansible_rulebook.exception import (
+    ConditionParsingException,
+    InvalidAssignmentException,
+    InvalidIdentifierException,
+    SelectattrOperatorException,
+    SelectOperatorException,
+)
 from ansible_rulebook.json_generator import (
     generate_dict_rulesets,
     visit_condition,
@@ -306,6 +313,177 @@ def test_parse_condition():
         parse_condition("fact.friends not contains 'fred'"), {}
     )
 
+    assert {
+        "SearchMatchesExpression": {
+            "lhs": {"Event": "url"},
+            "rhs": {
+                "SearchType": {
+                    "kind": {"String": "match"},
+                    "pattern": {
+                        "String": "https://example.com/users/.*/resources"
+                    },
+                    "options": [
+                        {
+                            "name": {"String": "ignorecase"},
+                            "value": {"Boolean": True},
+                        }
+                    ],
+                }
+            },
+        }
+    } == visit_condition(
+        parse_condition(
+            "event.url is "
+            + 'match("https://example.com/users/.*/resources", '
+            + "ignorecase=true)"
+        ),
+        {},
+    )
+
+    assert {
+        "SearchNotMatchesExpression": {
+            "lhs": {"Event": "url"},
+            "rhs": {
+                "SearchType": {
+                    "kind": {"String": "match"},
+                    "pattern": {
+                        "String": "https://example.com/users/.*/resources"
+                    },
+                    "options": [
+                        {
+                            "name": {"String": "ignorecase"},
+                            "value": {"Boolean": True},
+                        }
+                    ],
+                }
+            },
+        }
+    } == visit_condition(
+        parse_condition(
+            "event.url is not "
+            + 'match("https://example.com/users/.*/resources",ignorecase=true)'
+        ),
+        {},
+    )
+    assert {
+        "SearchMatchesExpression": {
+            "lhs": {"Event": "url"},
+            "rhs": {
+                "SearchType": {
+                    "kind": {"String": "regex"},
+                    "pattern": {"String": "example.com/foo"},
+                    "options": [
+                        {
+                            "name": {"String": "ignorecase"},
+                            "value": {"Boolean": True},
+                        }
+                    ],
+                }
+            },
+        }
+    } == visit_condition(
+        parse_condition(
+            'event.url is regex("example.com/foo",ignorecase=true)'
+        ),
+        {},
+    )
+
+    assert {
+        "SelectAttrExpression": {
+            "lhs": {"Event": "persons"},
+            "rhs": {
+                "key": {"String": "person.age"},
+                "operator": {"String": ">="},
+                "value": {"Integer": 50},
+            },
+        }
+    } == visit_condition(
+        parse_condition('event.persons is selectattr("person.age", ">=", 50)'),
+        {},
+    )
+
+    assert {
+        "SelectAttrExpression": {
+            "lhs": {"Event": "persons"},
+            "rhs": {
+                "key": {"String": "person.employed"},
+                "operator": {"String": "=="},
+                "value": {"Boolean": True},
+            },
+        }
+    } == visit_condition(
+        parse_condition(
+            'event.persons is selectattr("person.employed", "==", true)'
+        ),
+        {},
+    )
+
+    assert {
+        "SelectAttrNotExpression": {
+            "lhs": {"Event": "persons"},
+            "rhs": {
+                "key": {"String": "person.name"},
+                "operator": {"String": "=="},
+                "value": {"String": "fred"},
+            },
+        }
+    } == visit_condition(
+        parse_condition(
+            'event.persons is not selectattr("person.name", "==", "fred")'
+        ),
+        {},
+    )
+
+    assert {
+        "SelectExpression": {
+            "lhs": {"Event": "ids"},
+            "rhs": {"operator": {"String": ">="}, "value": {"Integer": 10}},
+        }
+    } == visit_condition(parse_condition('event.ids is select(">=", 10)'), {})
+
+    assert {
+        "SelectNotExpression": {
+            "lhs": {"Event": "persons"},
+            "rhs": {
+                "operator": {"String": "regex"},
+                "value": {"String": "fred|barney"},
+            },
+        }
+    } == visit_condition(
+        parse_condition('event.persons is not select("regex", "fred|barney")'),
+        {},
+    )
+
+    assert {
+        "SelectExpression": {
+            "lhs": {"Event": "is_true"},
+            "rhs": {"operator": {"String": "=="}, "value": {"Boolean": False}},
+        }
+    } == visit_condition(
+        parse_condition('event.is_true is select("==", False)'), {}
+    )
+
+
+def test_invalid_select_operator():
+    with pytest.raises(SelectOperatorException):
+        parse_condition('event.persons is not select("in", ["fred","barney"])')
+
+
+def test_invalid_selectattr_operator():
+    with pytest.raises(SelectattrOperatorException):
+        parse_condition(
+            'event.persons is not selectattr("name", "cmp", "fred")'
+        )
+
+
+def test_null_type():
+    assert {
+        "EqualsExpression": {
+            "lhs": {"Fact": "friend"},
+            "rhs": {"NullType": None},
+        }
+    } == visit_condition(parse_condition("fact.friend == null"), {})
+
 
 @pytest.mark.parametrize(
     "rulebook",
@@ -343,3 +521,40 @@ def test_generate_dict_ruleset(rulebook):
         ast = yaml.safe_load(f.read())
 
     assert ruleset == ast
+
+
+def test_invalid_assignment_operator():
+    with pytest.raises(InvalidAssignmentException):
+        visit_condition(
+            parse_condition("event.first << fact.range.i == 'Hello'"), {}
+        )
+
+
+def test_invalid_nested_assignment_operator():
+    with pytest.raises(InvalidAssignmentException):
+        visit_condition(
+            parse_condition("events.first.abc.xyz << fact.range.i == 'Hello'"),
+            {},
+        )
+
+
+def test_invalid_identifier():
+    with pytest.raises(InvalidIdentifierException):
+        visit_condition(
+            parse_condition("event is defined"),
+            {},
+        )
+
+
+@pytest.mark.parametrize(
+    "invalid_condition",
+    [
+        "event. is defined",
+    ],
+)
+def test_invalid_conditions(invalid_condition):
+    with pytest.raises(ConditionParsingException):
+        visit_condition(
+            parse_condition(invalid_condition),
+            {},
+        )
